/*
** wavwin_init.c for Gmurf in Gmurf/gui
** 
** Made by Rmi Coupet
** Login   <inthy@sietch.inthy.fr>
** 
** Started on  Thu Mar 11 13:35:41 1999 Rmi Coupet
** Last update Fri Mar 19 11:54:59 1999 Rmi Coupet
*/


#include "wavwin_init.h"
#include <stdio.h>

/* -------  PIXMAPS ----------  */
#include "xpms/pause_play.xpm"
#include "xpms/play.xpm"
#include "xpms/stop_play.xpm"
#include "xpms/sinus.xpm"
#include "xpms/histo.xpm"
#include "xpms/zoom_out.xpm"
#include "xpms/zoom_in.xpm"
#include "xpms/left.xpm"
#include "xpms/right.xpm"

/* -------  PIXMAPS ----------  */

void		toggle_histo		(GtkWidget	*widget,
					 GdkEvent	event,
					 gpointer	data) ;
void		toggle_curve		(GtkWidget	*widget,
					 GdkEvent	event,
					 gpointer	data) ;
void 		play_the_wav		(GtkWidget	*widget,
					 GdkEvent	*event,
					 gpointer 	data);
void 		init_chose_channel	(GtkWidget	*win_for_pix,
					 GtkWidget	*ww) ;
void		chose_left		(GtkWidget	*widget,
					 GdkEvent	*event ,
					 gpointer	data) ;
void		chose_right		(GtkWidget	*widget,
					 GdkEvent	*event ,
					 gpointer	data) ;
 void		export_in_mixer		(GtkWidget	*widget) ;
void		invert_button		(GtkWidget	*widget,
					 GdkEvent	event,
					 gpointer	data) ;


static ITEM representation[TOOLBAR_REPRESENTATION] = {
  {"Histogram",	toggle_histo, 	histo_xpm},
  {"Curve",	toggle_curve,	sinus_xpm} 
} ;

static ITEM play_buttons[WAVPLAYER_BUTTONS] = {
  {"Play",	NULL, play_xpm},
  {"Pause",	NULL, pause_play_xpm},
  {"Stop",	NULL, stop_play_xpm},
};

static ITEM navigation[NAVIGATION_BUTTONS] = {
  {"Zoom in",		zoom_in, zoom_in_xpm},
  {"Zoom out",		zoom_out, zoom_out_xpm},
  {"Zoom in Sel",	zoom_in_sel, zoom_in_xpm}
};

static ITEM chose[CHANNELS] = {
  {"left",	chose_left, 	left_xpm},
  {"right",	chose_right, 	right_xpm}
};


void 		new_wav_editor(tempgmurf)
t_tempgmurf	*tempgmurf ;
{
  GtkWidget 		*window ;
  GtkWidget 		*ww ;
  enum	{
    LEFT,
    RIGHT
  };
    
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL) ;
  gtk_widget_set_name(window, "wav editor") ;
  gtk_window_set_title(GTK_WINDOW (window), tempgmurf->origname) ;
  ww = wavwin_new() ;
  WAVWIN (ww)->m_win = window ;
  WAVWIN (ww)->tempgmurf = tempgmurf ;
  WAVWIN (ww)->playmode = (t_playmode*) malloc(sizeof(t_playmode));
  WAVWIN (ww)->playmode->chanels = CHANNELS ;
  gtk_signal_connect_object(GTK_OBJECT (window), "delete_event",
			    GTK_SIGNAL_FUNC (close_ww), 
			    GTK_OBJECT (ww));
  init_representation_buttons(WAVWIN (ww), MainWindow, representation, 
			      TOOLBAR_REPRESENTATION) ;
  init_wav_play_buttons(WAVWIN (ww), MainWindow, play_buttons, 
			WAVPLAYER_BUTTONS) ;
  init_navigation_buttons(WAVWIN (ww), MainWindow, navigation, 
			  NAVIGATION_BUTTONS) ;
  init_chose_channel(MainWindow, ww);

  create_wav_menu_bar(WAVWIN (ww), window) ;

  gtk_signal_connect_object(GTK_OBJECT (WAVWIN (ww)->export_button), "clicked",
			    GTK_SIGNAL_FUNC (export_in_mixer), GTK_OBJECT (ww)) ;
  gtk_signal_connect_object(GTK_OBJECT (WAVWIN (ww)->invert_button), "clicked",
			    GTK_SIGNAL_FUNC (invert_button), GTK_OBJECT (ww)) ;


  gtk_widget_hide(WAVEDITOR (WAVWIN (ww)->wve[VOLUME])->DrawingArea[LEFT]) ;
  gtk_widget_hide(WAVEDITOR (WAVWIN (ww)->wve[VOLUME])->DrawingArea[RIGHT]) ;
  gtk_widget_hide(WAVEDITOR (WAVWIN (ww)->wve[PANNING])->DrawingArea[LEFT]) ;
  gtk_widget_hide(WAVEDITOR (WAVWIN (ww)->wve[PANNING])->DrawingArea[RIGHT]) ;

  init_draw_funcs(WAVWIN (ww)) ;
  
  init_wve(WAVEDITOR (WAVWIN (ww)->wve[STANDARD]), WAVWIN (ww)) ;
  //  init_wve(WAVEDITOR (WAVWIN (ww)->wve[VOLUME])) ;
  //init_wve(WAVEDITOR (WAVWIN (ww)->wve[PANNING])) ;
  gtk_container_add(GTK_CONTAINER (window), ww) ;
  gtk_widget_show(ww) ;
  gtk_widget_show(window) ;
}


void 		connect_with_wav_editor(tempgmurf, ms)
t_tempgmurf	*tempgmurf ;
Mixer_Sample	*ms ;
{
  GtkWidget *window ;
  GtkWidget *ww ;
  enum	{
    LEFT,
    RIGHT
  };
    
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL) ;
  gtk_widget_set_name(window, "wav editor") ;
  gtk_window_set_title(GTK_WINDOW (window), tempgmurf->origname) ;
  ww = wavwin_new() ;
  WAVWIN (ww)->ms_connected = ms ;
  WAVWIN (ww)->m_win = window ;
  WAVWIN (ww)->tempgmurf = tempgmurf ;
  WAVWIN (ww)->playmode = (t_playmode*) malloc(sizeof(t_playmode));
  WAVWIN (ww)->playmode->chanels = CHANNELS ;
  gtk_signal_connect_object(GTK_OBJECT (window), "delete_event",
			    GTK_SIGNAL_FUNC (close_ww), 
			    GTK_OBJECT (ww));
  init_representation_buttons(WAVWIN (ww), MainWindow, representation, 
			      TOOLBAR_REPRESENTATION) ;
  init_wav_play_buttons(WAVWIN (ww), MainWindow, play_buttons, 
			WAVPLAYER_BUTTONS) ;
  init_navigation_buttons(WAVWIN (ww), MainWindow, navigation, 
			  NAVIGATION_BUTTONS) ;
  init_chose_channel(MainWindow, ww);

  create_wav_menu_bar(WAVWIN (ww), window) ;

  gtk_signal_connect_object(GTK_OBJECT (WAVWIN (ww)->export_button), "toggled",
			    GTK_SIGNAL_FUNC (export_in_mixer), GTK_OBJECT (ww)) ;
  gtk_signal_connect_object(GTK_OBJECT (WAVWIN (ww)->invert_button), "clicked",
			    GTK_SIGNAL_FUNC (invert_button), GTK_OBJECT (ww)) ;



  gtk_widget_hide(WAVEDITOR (WAVWIN (ww)->wve[VOLUME])->DrawingArea[LEFT]) ;
  gtk_widget_hide(WAVEDITOR (WAVWIN (ww)->wve[VOLUME])->DrawingArea[RIGHT]) ;
  gtk_widget_hide(WAVEDITOR (WAVWIN (ww)->wve[PANNING])->DrawingArea[LEFT]) ;
  gtk_widget_hide(WAVEDITOR (WAVWIN (ww)->wve[PANNING])->DrawingArea[RIGHT]) ;

  init_draw_funcs(WAVWIN (ww)) ;
  
  init_wve(WAVEDITOR (WAVWIN (ww)->wve[STANDARD]), WAVWIN (ww)) ;
  //  init_wve(WAVEDITOR (WAVWIN (ww)->wve[VOLUME])) ;
  //init_wve(WAVEDITOR (WAVWIN (ww)->wve[PANNING])) ;
  gtk_container_add(GTK_CONTAINER (window), ww) ;
  gtk_widget_show(ww) ;
  gtk_widget_show(window) ;
}


void		init_representation_buttons(ww,
					    win_for_pix,
					    represent,
					    nb_buttons)
Wavwin		*ww ;
GtkWidget	*win_for_pix ;
ITEM		*represent ;
int		nb_buttons ;
{
  int 	i ;
  
  for (i = 0; i < nb_buttons; i++)
  {
      gtk_container_add(GTK_CONTAINER (ww->toolbar_button[i]), 
			new_pix_from_data (MainWindow, represent[i].pix));
      gtk_signal_connect_object(GTK_OBJECT (ww->toolbar_button[i]), "toggled",
				GTK_SIGNAL_FUNC (represent[i].func), GTK_OBJECT (ww));
  }
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON 
				 (ww->toolbar_button[0]), TRUE);
  ww->histo_active = TRUE ;
}


void		init_navigation_buttons(ww,
					win_for_pix,
					navigate,
					nb_buttons)
Wavwin		*ww ;
GtkWidget	*win_for_pix ;
ITEM		*navigate ;
int		nb_buttons ;
{
  GtkWidget	*hbox ;
  GtkWidget	*label ;
  int		i ;
  
  for (i = 0; i < nb_buttons; i++) 
  {
    label = gtk_label_new(navigate[i].name) ;
    hbox = gtk_hbox_new(FALSE, 0) ;
    gtk_box_pack_start(GTK_BOX(hbox), 
		       new_pix_from_data (win_for_pix, navigate[i].pix),
		       FALSE, FALSE, 0);
    gtk_signal_connect_object(GTK_OBJECT (ww->navigation_button[i]), "clicked",
			      GTK_SIGNAL_FUNC (navigate[i].func), GTK_OBJECT (ww));
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    gtk_widget_show(label);
    gtk_container_add(GTK_CONTAINER (ww->navigation_button[i]), hbox) ;
    gtk_widget_show(hbox);
  }
}


void		init_wav_play_buttons(ww, 
				      win_for_pix,  
				      play_button,
				      nb_buttons)
			   
Wavwin		*ww ;
GtkWidget	*win_for_pix ;
ITEM		*play_button ;
int		nb_buttons ;
{
  gtk_signal_connect_object(GTK_OBJECT (ww->play_button[REMI_PLAY]), 
			    "clicked", GTK_SIGNAL_FUNC (play_the_wav), 
			    GTK_OBJECT (ww));
}

void 		play_the_wav(widget, event, data)
GtkWidget	*widget ;
GdkEvent	*event ;
gpointer 	data ;
{
  Wavwin	*ww ;
  t_select	*select;

  ww = WAVWIN(widget) ;
  select = create_select(WAVEDITOR(ww->wve[STANDARD])->selection.x,
			 WAVEDITOR(ww->wve[STANDARD])->selection.y - 
			 WAVEDITOR(ww->wve[STANDARD])->selection.x + 1, 
			 ww->invert, WAVEDITOR(ww->wve[STANDARD])->there_is_a_selection, 
			 ww->playmode->chanels) ;
  ww = WAVWIN (widget);
  if (gtk_notebook_current_page(GTK_NOTEBOOK (ww->notebook)) == STANDARD) {
    play_temp_file(ww->tempgmurf->file, 
		   MIXER(Global_Mixer)->gmurfrc->first_dsp->dsp->file,
		   WAVEDITOR (ww->wve[STANDARD])->part_watched.x,
		   WAVEDITOR (ww->wve[STANDARD])->part_watched.y -
		   WAVEDITOR (ww->wve[STANDARD])->part_watched.x,
		   ww->playmode, ww, select);
    free(select);
  }
}

void 		init_chose_channel(win_for_pix,
				   ww)
GtkWidget	*win_for_pix ;
GtkWidget	*ww ;
{
  Wavwin	*wav ;
  int		i ;
  
  wav = WAVWIN (ww) ;
  for (i=0; i < CHANNELS ; i++)
  {
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (wav->chose_channel[i]), 1);    
    gtk_container_add(GTK_CONTAINER (wav->chose_channel[i]),
		      new_pix_from_data (win_for_pix, chose[i].pix));
    gtk_signal_connect_object(GTK_OBJECT (wav->chose_channel[i]),
			      "toggled", GTK_SIGNAL_FUNC (chose[i].func),
			      (GTK_OBJECT (wav)));
  }
}


void 		toggle_histo(widget,
			     event,
			     data)
GtkWidget	*widget ;
GdkEvent	event ;
gpointer	data ;
{
  Wavwin	*ww ;
  
  ww = WAVWIN (widget) ;
  if (ww->histo_active == FALSE)
  {
    ww->histo_active = TRUE ;
    WAVEDITOR (ww->wve[STANDARD])->draw_wav_right = draw_wav_basic ;
    WAVEDITOR (ww->wve[STANDARD])->draw_wav_right(ww->wve[STANDARD], (gpointer) ww) ;
  } else 
    ww->histo_active = FALSE ;
}


void		invert_button(widget, event, data)
GtkWidget	*widget ;
GdkEvent	event ;
gpointer	data ;
{
  Wavwin	*ww ;
  
  ww = WAVWIN (widget) ;
  ww->invert = GTK_TOGGLE_BUTTON (ww->invert_button)->active ;
  gtk_signal_emit_by_name(GTK_OBJECT (ww->wve[STANDARD]), 
			  "channel_right_configure") ;
}


void 		toggle_curve(widget,
			     event,
			     data)
GtkWidget	*widget ;
GdkEvent	event ;
gpointer	data ;
{
  Wavwin	*ww ;
  
  ww = WAVWIN (widget) ;
  if (ww->histo_active == FALSE)
  {
    WAVEDITOR (ww->wve[STANDARD])->draw_wav_right = draw_wav_sinus ;
    WAVEDITOR (ww->wve[STANDARD])->draw_wav_right(ww->wve[STANDARD], (gpointer) ww) ;
  } 
}


void		chose_left(widget,
			   event,
			   data)
GtkWidget	*widget ;
GdkEvent	*event ;
gpointer	data ;
{
  Wavwin	*ww ;
  
  ww = WAVWIN (widget) ;

  if (GTK_TOGGLE_BUTTON (ww->chose_channel[LEFT])->active)
    if (GTK_TOGGLE_BUTTON (ww->chose_channel[RIGHT])->active)
      ww->playmode->chanels = CHANNELS ;
    else {
      ww->playmode->chanels = LEFT ;
    }
  else {
    ww->playmode->chanels = RIGHT ;
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ww->chose_channel[RIGHT]), TRUE);
  }
}



void		chose_right(widget,
			    event,
			    data)
GtkWidget	*widget ;
GdkEvent	*event ;
gpointer	data ;
{
  Wavwin	*ww ;
  
  ww = WAVWIN (widget) ;
  if (GTK_TOGGLE_BUTTON (ww->chose_channel[RIGHT])->active)
    if (GTK_TOGGLE_BUTTON (ww->chose_channel[LEFT])->active)
      ww->playmode->chanels = CHANNELS ;
    else	
      ww->playmode->chanels = RIGHT ;
  else
  {  
    ww->playmode->chanels = LEFT ;
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON (ww->chose_channel[LEFT]), TRUE);
  }
}

void		export_in_mixer(widget) 
GtkWidget	*widget ;
{
  Wavwin	*ww ;
  t_tempgmurf	*gmurf ;
  static int	i=0 ;
  char		*str ;
  int           destfd;
  int           result;
  char          buf[CPY_BUF_SIZE];
    
  ww = WAVWIN (widget) ;
  str = malloc(sizeof(char) * (strlen(ww->tempgmurf->tempname) 
			       + strlen(itoa(i)) + 1) ) ;
  strcpy(str, ww->tempgmurf->tempname) ;
  strcat(str, itoa(i)) ;  
  i++ ;

  if ((destfd = open(str, O_CREAT | O_TRUNC | O_WRONLY, 0644)) < 0)
    {
      printf("fuck! Opened failed.\n");      
    }
  else
  {	
    
    lseek(ww->tempgmurf->file, 0, SEEK_SET);
    while ((result = read(ww->tempgmurf->file, buf, CPY_BUF_SIZE)) > 0)
      write(destfd, buf, result);
    
    close(destfd);
    
    gmurf = new_tempgmurf(str, NULL);
    gmurf->temp_size = ww->tempgmurf->temp_size;
    strcpy(gmurf->origname, ww->tempgmurf->origname) ;
    /*gmurf = charge_sound_file(ww->tempgmurf->origname, str,
      MIXER (Global_Mixer)->gmurfrc->first_dsp->sample_rate) ;*/
    gmurf->connected = IN_MIXER ;
    Create_mix_sample(gmurf);	
  }
  
}


void		close_ww(widget) 
GtkWidget	*widget;
{
  Wavwin	*ww ;
  
  ww = WAVWIN(widget) ;  

  if (ww->tempgmurf->connected == IN_BOTH)
    ww->tempgmurf->connected = IN_MIXER ;
  else
  {
    
  }
  gtk_widget_destroy(ww->m_win) ;
}







